/*
 * RFID 读写卡测试程序 DFrobot
 *   MFRC522 模块
 *   MRC50-1 白卡
 * 模块资料：http://wiki.dfrobot.com.cn/index.php?title=RFID_Reader_Module%28.NET_Gadgeteer_Compatible%29_%28SKU:TOY0019%29
 */

/* 特别重要!! 
 * 如果使用的IDE版本小于1.0，请务必将下行注释掉
*/
#define NEWIDE 

unsigned char RevBuffer[30];
unsigned char CardID[4];//存储卡片序列号
unsigned char Read_Data[16]={0x00};
unsigned char PassWd[6]={0xFF,0xFF,0xFF,0xFF,0xFF,0xFF};//密码
unsigned char WriteData[16]={0};
unsigned char NewKey[16]={0x00};//存储新密码
unsigned char BlockN=0;
unsigned char Command;
unsigned char N;
unsigned char Wdata;


/////////////////////////////////////////////////////////////////////
//复位引脚定义
/////////////////////////////////////////////////////////////////////
#define RC522_RST  2//Dreamer MEGA  X14 port PIN6

#define SET_RC522RST  digitalWrite(RC522_RST,HIGH)
#define CLR_RC522RST  digitalWrite(RC522_RST,LOW)

/////////////////////////////////////////////////////////////////////
//MFRC522命令字
/////////////////////////////////////////////////////////////////////
#define PCD_IDLE              0x00               //取消当前命令
#define PCD_AUTHENT           0x0E               //验证密钥
#define PCD_RECEIVE           0x08               //接收数据
#define PCD_TRANSMIT          0x04               //发送数据
#define PCD_TRANSCEIVE        0x0C               //发送并接收数据
#define PCD_RESETPHASE        0x0F               //复位
#define PCD_CALCCRC           0x03               //CRC计算

/////////////////////////////////////////////////////////////////////
//Mifare_One卡片命令字
/////////////////////////////////////////////////////////////////////
#define PICC_REQIDL           0x26               //寻天线区内未进入休眠状态
#define PICC_REQALL           0x52               //寻天线区内全部卡
#define PICC_ANTICOLL1        0x93               //防冲撞
#define PICC_ANTICOLL2        0x95               //防冲撞
#define PICC_AUTHENT1A        0x60               //验证A密钥
#define PICC_AUTHENT1B        0x61               //验证B密钥
#define PICC_READ             0x30               //读块
#define PICC_WRITE            0xA0               //写块
#define PICC_DECREMENT        0xC0               //减值
#define PICC_INCREMENT        0xC1               //增值
#define PICC_RESTORE          0xC2               //调块数据到缓冲区
#define PICC_TRANSFER         0xB0               //保存缓冲区中数据
#define PICC_HALT             0x50               //休眠

/////////////////////////////////////////////////////////////////////
//MFRC522 FIFO长度定义
/////////////////////////////////////////////////////////////////////
#define DEF_FIFO_LENGTH       64                 //FIFO size=64byte
#define MAXRLEN  18

/////////////////////////////////////////////////////////////////////
//MFRC522寄存器定义
/////////////////////////////////////////////////////////////////////
// PAGE 0
#define     RFU00                 0x00    
#define     CommandReg            0x01    
#define     ComIEnReg             0x02    
#define     DivlEnReg             0x03    
#define     ComIrqReg             0x04    
#define     DivIrqReg             0x05
#define     ErrorReg              0x06    
#define     Status1Reg            0x07    
#define     Status2Reg            0x08    
#define     FIFODataReg           0x09
#define     FIFOLevelReg          0x0A
#define     WaterLevelReg         0x0B
#define     ControlReg            0x0C
#define     BitFramingReg         0x0D
#define     CollReg               0x0E
#define     RFU0F                 0x0F
// PAGE 1     
#define     RFU10                 0x10
#define     ModeReg               0x11
#define     TxModeReg             0x12
#define     RxModeReg             0x13
#define     TxControlReg          0x14
#define     TxAutoReg             0x15
#define     TxSelReg              0x16
#define     RxSelReg              0x17
#define     RxThresholdReg        0x18
#define     DemodReg              0x19
#define     RFU1A                 0x1A
#define     RFU1B                 0x1B
#define     MifareReg             0x1C
#define     RFU1D                 0x1D
#define     RFU1E                 0x1E
#define     SerialSpeedReg        0x1F
// PAGE 2    
#define     RFU20                 0x20  
#define     CRCResultRegM         0x21
#define     CRCResultRegL         0x22
#define     RFU23                 0x23
#define     ModWidthReg           0x24
#define     RFU25                 0x25
#define     RFCfgReg              0x26
#define     GsNReg                0x27
#define     CWGsCfgReg            0x28
#define     ModGsCfgReg           0x29
#define     TModeReg              0x2A
#define     TPrescalerReg         0x2B
#define     TReloadRegH           0x2C
#define     TReloadRegL           0x2D
#define     TCounterValueRegH     0x2E
#define     TCounterValueRegL     0x2F
// PAGE 3      
#define     RFU30                 0x30
#define     TestSel1Reg           0x31
#define     TestSel2Reg           0x32
#define     TestPinEnReg          0x33
#define     TestPinValueReg       0x34
#define     TestBusReg            0x35
#define     AutoTestReg           0x36
#define     VersionReg            0x37
#define     AnalogTestReg         0x38
#define     TestDAC1Reg           0x39  
#define     TestDAC2Reg           0x3A   
#define     TestADCReg            0x3B   
#define     RFU3C                 0x3C   
#define     RFU3D                 0x3D   
#define     RFU3E                 0x3E   
#define     RFU3F		  0x3F

/////////////////////////////////////////////////////////////////////
//和MFRC522通讯时返回的错误代码
/////////////////////////////////////////////////////////////////////
#define 	MI_OK                 0
#define 	MI_NOTAGERR           (-1)
#define 	MI_ERR                (-2)

/////////////////////////////////////////////////////////////////////
//功    能：读RC632寄存器
//参数说明：Address:寄存器地址
//返    回：读出的值
/////////////////////////////////////////////////////////////////////
unsigned char ReadRawRC(unsigned char Address)
{
    unsigned char ucAddr;
    unsigned char ucResult=0;

    ucAddr = (Address&0x3f)|0x80;//串口读地址格式

#ifndef NEWIDE	
    Serial1.print(ucAddr);
#else
    Serial1.write(ucAddr);
#endif

    delay(10);//等待
    if(Serial1.available() > 0) 
    ucResult=Serial1.read();

    return ucResult;
}
/////////////////////////////////////////////////////////////////////
//功    能：写RC632寄存器
//参数说明：Address:寄存器地址
//          value:写入的值
/////////////////////////////////////////////////////////////////////
void WriteRawRC(unsigned char Address, unsigned char value)
{  
    unsigned char ucAddr;
    unsigned char revAddr=0;

    ucAddr = Address&0x3f;//串口写地址格式

#ifndef NEWIDE	
    Serial1.print(ucAddr);
    Serial1.print(value);  
#else
    Serial1.write(ucAddr);
    Serial1.write(value);  
#endif

    delay(10);//等待返回确认地址
    
    while(Serial1.available() > 0) 
    revAddr=Serial1.read();
}
/////////////////////////////////////////////////////////////////////
//功    能：置RC522寄存器位
//参数说明：reg:寄存器地址
//          mask:置位值
/////////////////////////////////////////////////////////////////////
void SetBitMask(unsigned char reg,unsigned char mask)  
{
    char tmp = 0x00;
    tmp = ReadRawRC(reg);
    WriteRawRC(reg,tmp | mask);  // set bit mask
}
/////////////////////////////////////////////////////////////////////
//功    能：清RC522寄存器位
//参数说明：reg:寄存器地址
//          mask:清位值
/////////////////////////////////////////////////////////////////////
void ClearBitMask(unsigned char reg,unsigned char mask)  
{
    char tmp = 0x00;
    tmp = ReadRawRC(reg);
    WriteRawRC(reg, tmp & ~mask);  // clear bit mask
} 
/////////////////////////////////////////////////////////////////////
//开启天线  
//每次启动或关闭天险发射之间应至少有1ms的间隔
/////////////////////////////////////////////////////////////////////
void PcdAntennaOn(void)
{
    unsigned char i;
    i = ReadRawRC(TxControlReg);
    if (!(i & 0x03))
    {
        SetBitMask(TxControlReg, 0x03);
    }
}
/////////////////////////////////////////////////////////////////////
//关闭天线
/////////////////////////////////////////////////////////////////////
void PcdAntennaOff(void)
{
	ClearBitMask(TxControlReg, 0x03);
}
//////////////////////////////////////////////////////////////////////
//设置RC522的工作方式 
//////////////////////////////////////////////////////////////////////
char M500PcdConfigISOType(unsigned char type)
{
   if (type == 'A')                     //ISO14443_A
   { 
       ClearBitMask(Status2Reg,0x08);
       WriteRawRC(ModeReg,0x3D);
       WriteRawRC(RxSelReg,0x86);
       WriteRawRC(RFCfgReg,0x7F);
       WriteRawRC(TReloadRegL,30); 
       WriteRawRC(TReloadRegH,0);
       WriteRawRC(TModeReg,0x8D);
       WriteRawRC(TPrescalerReg,0x3E);
       delay(10);
       PcdAntennaOn();
   }
   else{ return -1; }
   
   return MI_OK;
}
/////////////////////////////////////////////////////////////////////
//功    能：复位RC522
//返    回: 成功返回MI_OK
/////////////////////////////////////////////////////////////////////
char PcdReset(void)
{
    SET_RC522RST;
    delay(10);
    CLR_RC522RST;
    delay(10);
    SET_RC522RST;
    delay(10);
    WriteRawRC(CommandReg,PCD_RESETPHASE);
    delay(10);
    
    WriteRawRC(ModeReg,0x3D);//和Mifare卡通讯，CRC初始值0x6363
    WriteRawRC(TReloadRegL,30);           
    WriteRawRC(TReloadRegH,0);
    WriteRawRC(TModeReg,0x8D);
    WriteRawRC(TPrescalerReg,0x3E);
	
    WriteRawRC(TxAutoReg,0x40);//必须要
   
    return MI_OK;
}
/////////////////////////////////////////////////////////////////////
//功    能：通过RC522和ISO14443卡通讯
//参数说明：Command:RC522命令字
//          pInData:通过RC522发送到卡片的数据
//          InLenByte:发送数据的字节长度
//          pOutData:接收到的卡片返回数据
//          *pOutLenBit:返回数据的位长度
/////////////////////////////////////////////////////////////////////
char PcdComMF522(unsigned char Command, 
                 unsigned char *pInData, 
                 unsigned char InLenByte,
                 unsigned char *pOutData, 
                 unsigned int *pOutLenBit)
{
    char stat = MI_ERR;
    unsigned char irqEn   = 0x00;
    unsigned char waitFor = 0x00;
    unsigned char lastBits;
    unsigned char n;
    unsigned int i;
    switch (Command)
    {
        case PCD_AUTHENT:
			irqEn   = 0x12;
			waitFor = 0x10;
			break;
		case PCD_TRANSCEIVE:
			irqEn   = 0x77;
			waitFor = 0x30;
			break;
		default:
			break;
    }
   
    WriteRawRC(ComIEnReg,irqEn|0x80);
    ClearBitMask(ComIrqReg,0x80);
    WriteRawRC(CommandReg,PCD_IDLE);
    SetBitMask(FIFOLevelReg,0x80);
    
    for (i=0; i<InLenByte; i++)
    {   WriteRawRC(FIFODataReg, pInData[i]);    }
    WriteRawRC(CommandReg, Command);
   
    
    if (Command == PCD_TRANSCEIVE)
    {    SetBitMask(BitFramingReg,0x80);  }
    
	i = 2000;
    do 
    {
        n = ReadRawRC(ComIrqReg);
        i--;
    }
    while ((i!=0) && !(n&0x01) && !(n&waitFor));
    ClearBitMask(BitFramingReg,0x80);

    if (i!=0)
    {    
        if(!(ReadRawRC(ErrorReg)&0x1B))
        {
            stat = MI_OK;
            if (n & irqEn & 0x01)
            {   stat = MI_NOTAGERR;   }
            if (Command == PCD_TRANSCEIVE)
            {
               	n = ReadRawRC(FIFOLevelReg);
              	lastBits = ReadRawRC(ControlReg) & 0x07;
                if (lastBits)
                {   *pOutLenBit = (n-1)*8 + lastBits;   }
                else
                {   *pOutLenBit = n*8;   }
                if (n == 0)
                {   n = 1;    }
                if (n > MAXRLEN)
                {   n = MAXRLEN;   }
                for (i=0; i<n; i++)
                {   pOutData[i] = ReadRawRC(FIFODataReg);    }
            }
        }
        else
        {   stat = MI_ERR;   }
        
    }
   

    SetBitMask(ControlReg,0x80);           // stop timer now
    WriteRawRC(CommandReg,PCD_IDLE); 
    return stat;
}
/////////////////////////////////////////////////////////////////////
//功    能：寻卡
//参数说明: req_code:寻卡方式
//                0x52 = 寻感应区内所有符合14443A标准的卡
//                0x26 = 寻未进入休眠状态的卡
//          pTagType：卡片类型代码
//                0x4400 = Mifare_UltraLight
//                0x0400 = Mifare_One(S50)
//                0x0200 = Mifare_One(S70)
//                0x0800 = Mifare_Pro(X)
//                0x4403 = Mifare_DESFire
//返    回: 成功返回MI_OK
/////////////////////////////////////////////////////////////////////
char PcdRequest(unsigned char req_code,unsigned char *pTagType)
{
	char stat;  
	unsigned int unLen;
	unsigned char ucComMF522Buf[MAXRLEN]; 

	ClearBitMask(Status2Reg,0x08);
	WriteRawRC(BitFramingReg,0x07);
	SetBitMask(TxControlReg,0x03);
 
	ucComMF522Buf[0] = req_code;

	stat = PcdComMF522(PCD_TRANSCEIVE,ucComMF522Buf,1,ucComMF522Buf,&unLen);

	if ((stat == MI_OK) && (unLen == 0x10))
	{    
		*pTagType     = ucComMF522Buf[0];
		*(pTagType+1) = ucComMF522Buf[1];
	}
	else
	{   stat = MI_ERR;   }
   
	return stat;
}
/////////////////////////////////////////////////////////////////////
//功    能：防冲撞
//参数说明: pSnr:卡片序列号，4字节
//返    回: 成功返回MI_OK
/////////////////////////////////////////////////////////////////////  
char PcdAnticoll(unsigned char *pSnr)
{
    char stat;
    unsigned char i,snr_check=0;
    unsigned int unLen;
    unsigned char ucComMF522Buf[MAXRLEN]; 
    

    ClearBitMask(Status2Reg,0x08);
    WriteRawRC(BitFramingReg,0x00);
    ClearBitMask(CollReg,0x80);
 
    ucComMF522Buf[0] = PICC_ANTICOLL1;
    ucComMF522Buf[1] = 0x20;

    stat = PcdComMF522(PCD_TRANSCEIVE,ucComMF522Buf,2,ucComMF522Buf,&unLen);

    if (stat == MI_OK)
    {
    	 for (i=0; i<4; i++)
         {   
             *(pSnr+i)  = ucComMF522Buf[i];
             snr_check ^= ucComMF522Buf[i];
         }
         if (snr_check != ucComMF522Buf[i])
         {   stat = MI_ERR;    }
    }
    
    SetBitMask(CollReg,0x80);
    return stat;
}
/////////////////////////////////////////////////////////////////////
//用MF522计算CRC16函数
/////////////////////////////////////////////////////////////////////
void CalulateCRC(unsigned char *pIndata,unsigned char len,unsigned char *pOutData)
{
    unsigned char i,n;
    ClearBitMask(DivIrqReg,0x04);
    WriteRawRC(CommandReg,PCD_IDLE);
    SetBitMask(FIFOLevelReg,0x80);
    for (i=0; i<len; i++)
    {   WriteRawRC(FIFODataReg, *(pIndata+i));   }
    WriteRawRC(CommandReg, PCD_CALCCRC);
    i = 0xFF;
    do 
    {
        n = ReadRawRC(DivIrqReg);
        i--;
    }
    while ((i!=0) && !(n&0x04));
    pOutData[0] = ReadRawRC(CRCResultRegL);
    pOutData[1] = ReadRawRC(CRCResultRegM);
}
/////////////////////////////////////////////////////////////////////
//功    能：选定卡片
//参数说明: pSnr:卡片序列号，4字节
//返    回: 成功返回MI_OK
/////////////////////////////////////////////////////////////////////
char PcdSelect(unsigned char *pSnr)
{
    char stat;
    unsigned char i;
    unsigned int unLen;
    unsigned char ucComMF522Buf[MAXRLEN]; 
    
    ucComMF522Buf[0] = PICC_ANTICOLL1;
    ucComMF522Buf[1] = 0x70;
    ucComMF522Buf[6] = 0;
    for (i=0; i<4; i++)
    {
    	ucComMF522Buf[i+2] = *(pSnr+i);
    	ucComMF522Buf[6]  ^= *(pSnr+i);
    }
    CalulateCRC(ucComMF522Buf,7,&ucComMF522Buf[7]);
  
    ClearBitMask(Status2Reg,0x08);

    stat = PcdComMF522(PCD_TRANSCEIVE,ucComMF522Buf,9,ucComMF522Buf,&unLen);
    
    if ((stat == MI_OK) && (unLen == 0x18))
    {   stat = MI_OK;  }
    else
    {   stat = MI_ERR;    }

    return stat;
}
/////////////////////////////////////////////////////////////////////
//功    能：验证卡片密码
//参数说明: auth_mode: 密码验证模式
//                 0x60 = 验证A密钥
//                 0x61 = 验证B密钥 
//          addr：块地址
//          pKey：密码
//          pSnr：卡片序列号，4字节
//返    回: 成功返回MI_OK
/////////////////////////////////////////////////////////////////////               
char PcdAuthState(unsigned char auth_mode,unsigned char addr,unsigned char *pKey,unsigned char *pSnr)
{
    char stat;
    unsigned int unLen;
    unsigned char i,ucComMF522Buf[MAXRLEN]; 

    ucComMF522Buf[0] = auth_mode;
    ucComMF522Buf[1] = addr;
    for (i=0; i<6; i++)
    {    ucComMF522Buf[i+2] = *(pKey+i);   }
    for (i=0; i<6; i++)
    {    ucComMF522Buf[i+8] = *(pSnr+i);   }
    
    stat = PcdComMF522(PCD_AUTHENT,ucComMF522Buf,12,ucComMF522Buf,&unLen);
    if ((stat != MI_OK) || (!(ReadRawRC(Status2Reg) & 0x08)))
    {   stat = MI_ERR;   }
    
    return stat;
}
/////////////////////////////////////////////////////////////////////
//功    能：读取M1卡一块数据
//参数说明: addr：块地址
//          pData：读出的数据，16字节
//返    回: 成功返回MI_OK
///////////////////////////////////////////////////////////////////// 
char PcdRead(unsigned char addr,unsigned char *pData)
{
    char stat;
    unsigned int unLen;
    unsigned char i,ucComMF522Buf[MAXRLEN]; 

    ucComMF522Buf[0] = PICC_READ;
    ucComMF522Buf[1] = addr;
    CalulateCRC(ucComMF522Buf,2,&ucComMF522Buf[2]);
   
    stat = PcdComMF522(PCD_TRANSCEIVE,ucComMF522Buf,4,ucComMF522Buf,&unLen);
    if ((stat == MI_OK) && (unLen == 0x90))
    {
        for (i=0; i<16; i++)
        {    *(pData+i) = ucComMF522Buf[i];   }
    }
    else
    {   stat = MI_ERR;   }
    
    return stat;
}
/////////////////////////////////////////////////////////////////////
//功    能：写数据到M1卡一块
//参数说明: addr：块地址
//          pData：写入的数据，16字节
//返    回: 成功返回MI_OK
/////////////////////////////////////////////////////////////////////                  
char PcdWrite(unsigned char addr,unsigned char *pData)
{
    char stat;
    unsigned int unLen;
    unsigned char i,ucComMF522Buf[MAXRLEN]; 
    
    ucComMF522Buf[0] = PICC_WRITE;
    ucComMF522Buf[1] = addr;
    CalulateCRC(ucComMF522Buf,2,&ucComMF522Buf[2]);
 
    stat = PcdComMF522(PCD_TRANSCEIVE,ucComMF522Buf,4,ucComMF522Buf,&unLen);

    if ((stat != MI_OK) || (unLen != 4) || ((ucComMF522Buf[0] & 0x0F) != 0x0A))
    {   stat = MI_ERR;   }
        
    if (stat == MI_OK)
    {
        for (i=0; i<16; i++)
        {    
        	ucComMF522Buf[i] = *(pData+i);   
        }
        CalulateCRC(ucComMF522Buf,16,&ucComMF522Buf[16]);

        stat = PcdComMF522(PCD_TRANSCEIVE,ucComMF522Buf,18,ucComMF522Buf,&unLen);
        if ((stat != MI_OK) || (unLen != 4) || ((ucComMF522Buf[0] & 0x0F) != 0x0A))
        {   stat = MI_ERR;   }
    }
    
    return stat;
}
/////////////////////////////////////////////////////////////////////
//功    能：命令卡片进入休眠状态
//返    回: 成功返回MI_OK
/////////////////////////////////////////////////////////////////////
char PcdHalt(void)
{
    char stat;
    unsigned int unLen;
    unsigned char ucComMF522Buf[MAXRLEN]; 

    ucComMF522Buf[0] = PICC_HALT;
    ucComMF522Buf[1] = 0;
    CalulateCRC(ucComMF522Buf,2,&ucComMF522Buf[2]);
 
    stat = PcdComMF522(PCD_TRANSCEIVE,ucComMF522Buf,4,ucComMF522Buf,&unLen);

    return MI_OK;
}


/////////////////////////////////////////////////////////////////////
//功    能：初始化RC522
//
/////////////////////////////////////////////////////////////////////
void InitRc522(void)
{
  PcdReset(); 
  PcdAntennaOff();  
  PcdAntennaOn();
  M500PcdConfigISOType( 'A' );
}
/////////////////////////////////////////////////////////////////////
//功    能：操作MFRC522
/////////////////////////////////////////////////////////////////////
void CtrlRc522(void)
{
     char stat;
     unsigned char ii;
     
     if(Command=='Q')//寻卡
     {
           PcdReset();
           delay(100);
           
           stat=PcdRequest(PICC_REQIDL,&RevBuffer[0]);//寻天线区内未进入休眠状态的卡，返回卡片类型 2字节
           if(stat!=MI_OK)
           {
            return;
           }
           //发送卡片类型代码
           Serial.print("Card type code; ");
           Serial.print(RevBuffer[0],DEC);
           Serial.print(',');
           Serial.println(RevBuffer[1],DEC);
           Serial.println();
     }
     else if(Command=='A')//防冲撞及选卡
     {
           stat=PcdAnticoll(&RevBuffer[2]);//防冲撞，返回卡的序列号 4字节
           if(stat!=MI_OK)
           {
              return;
           }
           memcpy(CardID,&RevBuffer[2],4); 
           stat=PcdSelect(CardID);//选卡
           if(stat!=MI_OK)
           {
               return;
           }
           //发送卡片序列号
           Serial.print("Card series number; ");
           Serial.print(CardID[0],DEC);
           Serial.print(',');
           Serial.print(CardID[1],DEC);
           Serial.print(',');
           Serial.print(CardID[2],DEC);
           Serial.print(',');
           Serial.println(CardID[3],DEC);
           Serial.println();
     }
     else if(Command=='R' && (BlockN>=0 && BlockN<=15))//读卡
     {
           stat=PcdAuthState(PICC_AUTHENT1A,BlockN,PassWd,CardID);//验证密码
           if(stat!=MI_OK)
           {
               return;
           }
           stat=PcdRead(BlockN,Read_Data);//读取块N数据
           if(stat!=MI_OK)
           {
               return;
           }
           //PcdHalt();//卡片休眠，不响应读写模块的任何命令；可使用寻卡命令唤醒。
           //发送卡片块N的内容
           Serial.print("Card block");
           Serial.print(BlockN,DEC);
           Serial.print(" content; ");
           for(ii=0;ii<15;ii++)
           {
               Serial.print(Read_Data[ii],DEC);
               Serial.print(',');
           }
           Serial.println(Read_Data[15],DEC);
           Serial.println();
     }
     else if(Command=='W' && ((BlockN>0 && BlockN<15) && BlockN!=3 && BlockN!=7 && BlockN!=11))//写卡
     {
           stat=PcdAuthState(PICC_AUTHENT1A,BlockN,PassWd,CardID);//验证密码
           if(stat!=MI_OK)
           {
               return;
           }
           stat=PcdWrite(BlockN,&WriteData[0]);//向块N写数据
           if(stat!=MI_OK)
           {
               return;
           }	
           //PcdHalt();//卡片休眠，不响应读写模块的任何命令；可使用寻卡命令唤醒。
           //发送卡片块N写OK
           Serial.print("Card block");
           Serial.print(BlockN,DEC);
           Serial.println(" Write ok!");
           Serial.println();
     }  
     //密码A永远不可读，读出数据显示为“Ox000000”,因此一旦设定就必须记住；一定条件下可改写，详见手册
     //密码B在一定条件下可读写，详见手册
    /*else if(Command=='K')//修改密码
     {   
           stat=PcdAuthState(PICC_AUTHENT1A,BlockN,PassWd,CardID);//验证密码
           if(stat!=MI_OK)
           {
               return;
           }
           stat=PcdWrite(BlockN,&NewKey[0]);//写新密码
           if(stat!=MI_OK)
           {
               return;
           }
           //PcdHalt();//卡片休眠，不响应读写模块的任何命令；可使用寻卡命令唤醒。		
     }*/

}
void setup()
{
     //设定端口模式
     pinMode(RC522_RST,OUTPUT);
     SET_RC522RST;
     
     //串口初始化
     Serial1.begin(9600);//串口2：主控板与模块通信
     Serial.begin(9600);//串口：主控板与USB通信
     delay(1000);
          
     InitRc522();//初始化
     delay(100);
     Serial.println("The initialization completes.  ");
     Serial.println();
}
void loop()
{    
         if(Serial.available() > 0) 
         {
             Command=Serial.read();//接收命令字符
             delay(10);
             if(Command=='R'|| Command=='W')
             {
                   N=Serial.read();
                   
                   if(Command=='W')
                   {      
                      for(char i=0;i<16;i++)
                      {
                        if(Serial.available() > 0)
                        {
                          delay(10);
                          Wdata=Serial.read();
                          Wdata -='0'; 
                          WriteData[i]=Wdata;
                        }
                        else WriteData[i]=0;
                      }
                   }
                   
                   switch(N)
                   {
                     case '0': BlockN=0;
                               break;
                     case '1': BlockN=1;
                               break;
                     case '2': BlockN=2;
                               break;   
                     case '3': BlockN=3;
                               break;
                     case '4': BlockN=4;
                               break;
                     case '5': BlockN=5;
                               break;   
                     case '6': BlockN=6;
                               break;
                     case '7': BlockN=7;
                               break;
                     case '8': BlockN=8;
                               break; 
                     case '9': BlockN=9;
                               break;
                     case 'A': BlockN=10;
                               break;
                     case 'B': BlockN=11;
                               break;
                     case 'C': BlockN=12;
                               break;
                     case 'D': BlockN=13;
                               break;
                     case 'E': BlockN=14;
                               break;  
                     case 'F': BlockN=15;
                               break;                             
                     default:  Serial.println("Error!");
                               Serial.println();
                               BlockN=16;
                   }
             }
             CtrlRc522();//操作MFRC522
         }
     
     
}

